#!/usr/bin/env python
# -*- coding:utf-8 -*-
import platform
from locale import getpreferredencoding
from pathlib import Path
from subprocess import PIPE, Popen
from time import time

from ..common import strings
from ..exceptions import TimeoutExpiredException
from ..log.factory import _LoggerFactory

_logger = _LoggerFactory().get_logger("CommandlineExec")
_encoding = getpreferredencoding(False)


class CommandlineExec(object):
    def __init__(self, process: Popen, timeout: int):
        self.__process: Popen = process
        self.__timeout: int = timeout
        self.__out: str = ""
        self.__err: str = ""
        self.__code: int = -1

    def complete(self) -> 'CommandlineExec':
        t1 = time()
        while self.__process.poll() is None:
            if time() - t1 > self.__timeout:
                self.__process.kill()
                raise TimeoutExpiredException("exec commandline timeout!!!")
        self.__out = self.__process.stdout.read()
        self.__err = self.__process.stderr.read()
        self.__code = self.__process.returncode
        log = f"\nexit code => {self.__code}\n" \
              f"out => {self.__out}\n" \
              f"err => {self.__err}\n" \
              f"{'CommandlineExec END'.center(51, '*')}"
        _logger.info(log)
        return self

    def input(self, content: str) -> 'CommandlineExec':
        if not isinstance(content, str):
            raise TypeError("need str type params")
        self.__process.stdin.write(content)
        self.__process.stdin.write("\n")
        self.__process.stdin.flush()
        return self

    @staticmethod
    def exec(cmd, cwd: str or Path = Path.cwd(), timeout: int = 60,
             encoding: str = _encoding, **kwargs) -> 'CommandlineExec':
        """
        Execute the entry
        :param encoding: encode
        :param cmd: commandline
        :param cwd: working directory
        :param timeout: timeout, default 60 second
        """
        log = f"\n{'CommandlineExec START'.center(49, '*')}\n" \
              f"OS: {platform.system()}\n" \
              f"Terminal Encoding: {encoding}\n"
        if cwd:
            log += f"working directory => {cwd}\n"
        log += f"commandline => {cmd}\n"
        _logger.info(log)
        process = Popen(cmd, cwd=cwd, shell=True, universal_newlines=True, encoding=encoding,
                        stdin=PIPE, stdout=PIPE, stderr=PIPE, **kwargs)
        return CommandlineExec(process, timeout)

    @property
    def code(self) -> int:
        """
        Returns the exit code after executing the command
        """
        return self.__code

    @property
    def out(self) -> str:
        """
        Returns standard output
        """
        return self.__out

    @property
    def err(self) -> str:
        """
        Returns standard error
        """
        return self.__err

    @property
    def is_success(self) -> bool:
        """
        Determine whether the command line is executed successfully
        """
        return self.__code == 0

    @property
    def is_fail(self) -> bool:
        """
        Determine whether the command line is executed failed
        """
        return not self.is_success

    @property
    def out_is_empty(self) -> bool:
        """
        Determine whether the standard output is blank
        """
        return strings.is_empty(self.__out)

    @property
    def out_is_not_empty(self) -> bool:
        """
        Judge that the standard output is not blank
        """
        return not self.out_is_empty

    def out_contain(self, value: str) -> bool:
        """
        Judge that the standard output contains a value
        """
        return strings.contains(self.__out, value)

    def out_not_contain(self, value: str) -> bool:
        """
        The criterion output does not contain a value
        """
        return not self.out_contain(value)

    def out_trip_contain(self, value: str) -> bool:
        """
        Judgment standard output contains values (after removing leading and trailing spaces)
        """
        return strings.trip_contains(self.__out, value)

    def out_trip_not_contain(self, value: str) -> bool:
        """
        Judgment standard output does not contain values (after removing leading and trailing spaces)
        """
        return not self.out_trip_contain(value)

    def out_equal(self, value: str) -> bool:
        """
        Judge that the standard output is equal to the value
        """
        return value == self.__out

    def out_not_equal(self, value: str) -> bool:
        """
        Determine that the standard output is not equal to the value
        """
        return not self.out_equal(value)

    def out_trip_equal(self, value: str) -> bool:
        """
        Judge that the standard output is equal to the value (after removing the leading and trailing spaces)
        """
        return value.strip() == self.__out.strip()

    def out_trip_not_equal(self, value: str) -> bool:
        """
        Judge that the standard output is not equal to the value (after removing the leading and trailing spaces)
        """
        return not self.out_trip_equal(value)

    @property
    def err_is_empty(self) -> bool:
        """
        The criterion error is blank
        """
        return strings.is_empty(self.__err)

    @property
    def err_is_not_empty(self) -> bool:
        """
        Criterion errors are not blank
        """
        return not self.err_is_empty

    def err_contain(self, value: str) -> bool:
        """
        The criterion error contains a value
        """
        return strings.contains(self.__err, value)

    def err_not_contain(self, value: str) -> bool:
        """
        The criterion error does not contain a value
        """
        return not self.err_contain(value)

    def err_trip_contain(self, value: str) -> bool:
        """
        Judgment criterion error contains value (after removing leading and trailing spaces)
        """
        return strings.trip_contains(self.__err, value)

    def err_trip_not_contain(self, value: str) -> bool:
        """
        Criterion error does not contain a value (after removing leading and trailing spaces)
        """
        return not self.err_trip_contain(value)

    def err_equal(self, value: str) -> bool:
        """
        The criterion error is equal to the value
        """
        return value == self.__err

    def err_not_equal(self, value: str) -> bool:
        """
        The criterion error is not equal to the value
        """
        return not self.err_equal(value)

    def err_trip_equal(self, value: str) -> bool:
        """
        The criterion error is equal to the value (after removing the leading and trailing spaces)
        """
        return value.strip() == self.__err.strip()

    def err_trip_not_equal(self, value: str) -> bool:
        """
        The criterion error is not equal to the value (after removing the leading and trailing spaces, it is judged)
        """
        return not self.err_trip_equal(value)


__all__ = ['CommandlineExec']
