#!/usr/bin/env python
# -*- coding:utf-8 -*-
import socket
import uuid
from typing import KeysView

from psutil import *
from psutil._common import scpufreq as cpufreq
from psutil._common import scpustats as cpustatus
from psutil._common import sswap as swap
from psutil._pswindows import scputimes as cputimes
from psutil._pswindows import svmem as vmem


class Network(object):
    """
    Network information
    """

    def __init__(self):
        self.__host__name: str = self.__get_host_name()
        self.__ip: str = self.__get_ip()
        self.__mac: str = self.__get_mac()
        self.__addresses: dict = net_if_addrs()
        self.__network_adapter_names: KeysView = self.__addresses.keys()

    @property
    def host_name(self) -> str:
        """
        get hostname
        """
        return self.__host__name

    @property
    def ip(self) -> str:
        """
        Get the IP address
        """
        return self.__ip

    @property
    def mac(self) -> str:
        """
        Get the MAC address
        Applies to a single NIC, if multiple NICs may be inconsistent with expectations
        """
        return self.__mac

    @property
    def network_adapter_names(self) -> list:
        """
        Gets the names of all network adapters
        """
        return list(net_if_addrs().keys())

    def mac_by_adapter_name(self, name: str) -> str:
        """
        Get the MAC by the name of the network adapter
        """
        return self.__get_mac_by_adapter_name(name)

    def ipv4_by_adapter_name(self, name: str) -> str:
        """
        Get IPv4 by the name of the network adapter
        """
        return self.__get_ipv4_by_adapter_name(name)

    def ipv6_by_adapter_name(self, name: str) -> str:
        """
        Get IPv6 by the name of the network adapter
        """
        return self.__get_ipv6_by_adapter_name(name)

    @staticmethod
    def __get_host_name() -> str:
        return socket.getfqdn(socket.gethostname())

    @staticmethod
    def __get_ip() -> str:
        return socket.gethostbyname(Network.__get_host_name())

    @staticmethod
    def __get_mac() -> str:
        mac_ = uuid.UUID(int=uuid.getnode()).hex[-12:].upper()
        return "-".join([mac_[e:e + 2] for e in range(0, 11, 2)])

    def __get_mac_by_adapter_name(self, name: str) -> str:
        return self.__get_info_by_adapter_name_and_family(name, {'AF_LINK', 'AF_PACKET'})

    def __get_ipv4_by_adapter_name(self, name: str) -> str:
        return self.__get_info_by_adapter_name_and_family(name, "AF_INET")

    def __get_ipv6_by_adapter_name(self, name: str) -> str:
        return self.__get_info_by_adapter_name_and_family(name, "AF_INET6")

    @staticmethod
    def __get_info_by_adapter_name_and_family(name: str, family: str or dict) -> str:
        addresses = net_if_addrs()
        for adapter_ in addresses:
            if adapter_ == name:
                snic_list = addresses[name]
                for snic in snic_list:
                    if snic.family.name in family:
                        return snic.address


class Cpu(object):
    """CPU information"""

    @property
    def count(self) -> int:
        """
        The number of physical CPUs
        """
        return cpu_count(False)

    @property
    def count_logical(self) -> int:
        """
        The number of logical CPUs
        """
        return cpu_count()

    def percent(self, interval: int = None) -> float:
        """
        Get the usage of each CPU
        :param interval: The time interval at which the data is fetched
        """
        return cpu_percent(interval, False)

    def percent_all(self, interval: int = None) -> float:
        """
        Gets the total CPU usage
        :param interval: The time interval at which the data is fetched
        """
        return cpu_percent(interval, True)

    def times(self) -> list[cputimes]:
        """
        获Get the usage of each CPU
        """
        return cpu_times(True)

    def times_all(self) -> cputimes:
        """
        The total time it takes to get the CPU
        """
        return cpu_times(False)

    def times_percent(self, interval: int = None) -> cputimes:
        """
        Get the percentage of total CPU time consumed
        :param interval: The time interval at which the data is fetched
        """
        return cpu_times_percent(interval, False)

    def times_percent_all(self, interval: int = None) -> list[cputimes]:
        """
        Get the proportion of time spent per CPU
        :param interval: The time interval at which the data is fetched
        """
        return cpu_times_percent(interval, True)

    def status(self) -> cpustatus:
        """
        Returns CPU statistics in the form of named tuples, including context switches, interrupts, soft interrupts,
        and system calls.
        """
        return cpu_stats()

    def freq(self) -> cpufreq:
        """
        Total CPU frequency
        """
        return cpu_freq(False)

    def freq_all(self) -> list[cpufreq]:
        """
        The frequency of each CPU
        """
        return cpu_freq(True)


class Memory(object):

    @staticmethod
    def virtual() -> vmem:
        """
        Get virtual memory information
        """
        return virtual_memory()

    @staticmethod
    def swap() -> swap:
        """
        Get the swap memory information
        """
        return swap_memory()


class Process(object):
    def pids(self) -> list[int]:
        """
        Returns the currently running processes as a list
        """
        return pids()

    def pid_exists(self, pid: int) -> bool:
        """
        Determine whether the given PID exists
        """
        return pid_exists(pid)

    def process_iter(self, attrs: list[str] = None) -> list:
        """
        Iterates over the currently running process, returning the Process object for each process
        :param attrs: The process property can be filtered and is a list
        """
        return list(process_iter(attrs))

    def real_process(self, pid: int = None, pname: str = None) -> Process:
        """
        Get process information
        :param pid: When the process ID and pname exist at the same time, only pid takes effect
        :param pname: process's name
        """
        for p in process_iter():
            if pid is not None and p.pid == pid:
                return p
            elif pname and p.name() == pname:
                return p

    def cmdline(self, pid: int = None, pname: str = None) -> str:
        """
        Gets the command-line arguments that start the process
        :param pid: When the process ID and pname exist at the same time, only pid takes effect
        :param pname: process's name
        """
        p = self.real_process(pid, pname)
        if p:
            return p.cmdline()

    def create_time(self, pid: int = None, pname: str = None) -> str:
        """
        Get the creation time of the process (timestamp format)
        :param pid: When the process ID and pname exist at the same time, only pid takes effect
        :param pname: process's name
        """
        p = self.real_process(pid, pname)
        if p:
            return p.create_time()

    def num_fds(self, pid: int = None, pname: str = None) -> int:
        """
        The number of files open by the process
        :param pid: When the process ID and pname exist at the same time, only pid takes effect
        :param pname: process's name
        """
        p = self.real_process(pid, pname)
        if p:
            return p.num_fds()

    def num_threads(self, pid: int = None, pname: str = None) -> int:
        """
        The number of child processes of the process
        :param pid: When the process ID and pname exist at the same time, only pid takes effect
        :param pname: process's name
        """
        p = self.real_process(pid, pname)
        if p:
            return p.num_threads()

    def is_running(self, pid: int = None, pname: str = None) -> bool:
        """
        Determine if the process is running
        :param pid: When the process ID and pname exist at the same time, only pid takes effect
        :param pname: process's name
        """
        p = self.real_process(pid, pname)
        if p:
            return p.is_running()

    def send_signal(self, pid: int = None, pname: str = None, kill_signal: int = 2):
        """
        Send signals to processes, similar to os.kill, etc
        :param pid: When the process ID and pname exist at the same time, only pid takes effect
        :param pname: process's name
        :param kill_signal: kill signal
        """
        p = self.real_process(pid, pname)
        if p:
            return p.send_signal(kill_signal)

    def kill(self, pid: int = None, pname: str = None):
        """
        Send a SIGKILL signal to end the process
        :param pid: When the process ID and pname exist at the same time, only pid takes effect
        :param pname: process's name
        """
        p = self.real_process(pid, pname)
        if p:
            return p.send_signal(9)

    def terminate(self, pid: int = None, pname: str = None):
        """
        Send a SIGTEAM signal to end the process
        :param pid: When the process ID and pname exist at the same time, only pid takes effect
        :param pname: process's name
        """
        p = self.real_process(pid, pname)
        if p:
            return p.send_signal(15)


network = Network()
cpu = Cpu
memory = Memory()
process = Process()

__all__ = ["network", "cpu", "memory", "process"]
