#!/usr/bin/env python
# -*- coding:utf-8 -*-
from functools import wraps
from typing import Dict, List

from ._around_impl import around_impl


def around(before: Dict[callable, dict] or List[callable] or callable = None,
           after: Dict[callable, dict] or List[callable] or callable = None,
           catch: bool = False, deliver: bool = True):
    """
    Preform facet operations on functions
    It supports injecting the return value of the preceding hook function into the decorated function
    Support to inject the return value of the decorated function into the post hook function.

    The decorated function can get the return value of the fore hook function through the "func_return" parameter,
    and the after hook function can get the return value of the decorated function via the "func_return" parameter.

    For example:
                def before():
                    return {"a": 1}


                def before2(func_return):
                    assert func_return["a"] == 1
                    return {"a": 2}


                def after(a, *args, b, func_return, c=1, **kwargs):
                    assert func_return["test_case1"]["a"] == 3
                    return {"a": 4}


                def after2(a, *args, b, func_return, c=1, **kwargs):
                    assert func_return["a"] == 3


                @around(before={before: {}, before2: {}}, after={after: {}})
                def test_case1(a, b, *c, func_return=None):
                    assert func_return["before"]["a"] == 1
                    assert func_return["before2"]["a"] == 2
                    return {"a": 3}


                @around(before=before, after={after2: {}}, deliver=True)
                def test_case2(a, b, *c, func_return=None):
                    assert func_return["a"] == 1
                    return {"a": 3}


                if __name__ == '__main__':
                    test_case1({"x": 3})
                    test_case2({"x": 3})


    :param deliver:
                    True: 'func_return' params only have previous  function return value.
                    False: 'func_return' params have previous all function return value.
    :param catch: decorated function throw exception when runtime, if True, will catch exception and run hook function,
                    then throw origin exception. If False, throw the exception directly.
                    Valid only for after hook functions.
    :param before:
        Preceding hook function before the decorated function is executed.
        If "before" is a dictionary, the key is the hook function object,
        and the value is the parameter of the hook function.
        When the hook function is executed, it will be passed to the hook function in the form of key value pairs.
        If "before" is a list, it means that the hook function has no parameters.
        If "before" is an executable object, the hook function is directly executed
    :param after:
        Post hook function.
        reference resources @params before
    """

    def _inner(func):
        @wraps(func)
        def _wrapper(*args: tuple, **kwargs: dict):
            around_impl(func, before, after, catch, deliver, *args, **kwargs)
        return _wrapper
    return _inner
