#!/usr/bin/env python
# -*- coding:utf-8 -*-
from functools import wraps
from inspect import getfullargspec

from typing import Type

from ..exceptions import CallException


def capture(exception: Type[Exception] = Exception, message: str = "", call: callable = None):
    """
    Catch exceptions to decorated functions, including exceptions generated by other decorators.
    :param call: while callback function when happen exception
    :param message: custom exception message
    :param exception: The base type exception to the exception that needs to be caught. After capture,
                    the exception will be "eaten" directly, and exception outside this range will be thrown.
    :return:
    """

    def _inner(func):
        @wraps(func)
        def _wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except BaseException as e:
                if issubclass(type(e), exception):
                    if message:
                        raise exception(message)
                    raise
            finally:
                if callable(call):
                    func_origin_args = getfullargspec(call).args
                    tmp_args = copy(args, func)
                    for i in range(len(tmp_args)):
                        kwargs[func_origin_args[i]] = tmp_args[i]
                    try:
                        call(*tmp_args, **kwargs)
                    except Exception as tmp_e:
                        raise CallException(f"call back exception: {str(tmp_e)}")

        return _wrapper

    return _inner


def copy(values: tuple, func: callable) -> tuple:
    class_name = func.__qualname__.split(".")[0]
    tmp = []
    tmp_append = tmp.append
    for v in values:
        if v.__class__.__name__ == class_name:
            continue
        tmp_append(v)
    return tuple(tmp)
