#!/usr/bin/env python
# -*- coding:utf-8 -*-
import platform
import sys
from datetime import datetime
from logging import StreamHandler, Formatter, Logger
from logging.handlers import RotatingFileHandler
from pathlib import Path

from ..log import LogConfig


class _LoggerWrapper(Logger):
    def __init__(self, name: str):
        super().__init__(name, LogConfig.level.value)
        self.__initialize()

    def __initialize(self):
        log_dir: Path = Path(LogConfig.dir)
        if not log_dir.exists():
            log_dir.mkdir(parents=True)
        for handler_type in ["ch", "fh"]:
            self.addHandler(self.__handler__(handler_type))
        self.setLevel(LogConfig.level.value)

    @staticmethod
    def __handler__(handlerType: str):
        if "ch" == handlerType:
            handler = StreamHandler()
        elif "fh" == handlerType:
            handler = RotatingFileHandler(filename=LogConfig.path,
                                          encoding=LogConfig.code,
                                          maxBytes=LogConfig.max_bytes,
                                          backupCount=LogConfig.backup_count)
        else:
            raise Exception(f"unknown handler type: {handlerType}")

        handler.setLevel(LogConfig.level.value)
        handler.setFormatter(Formatter(LogConfig.format))
        return handler


class _LoggerFactory(object):

    def __init__(self, banner: bool = False):
        self.__logger_dict = {}
        self.__show_banner(banner)

    def __show_banner(self, banner: bool = False):
        if not banner:
            return
        if not LogConfig.banner:
            return
        init_len = 100
        split_len = 16
        key_len = 16
        value_len = 0
        left_len = 4
        logo = 'INITIALIZE LOG ENVIRONMENT'
        logo_len = len(logo)
        content = f"""
{''.center(init_len, "#")}
{'#'.ljust(int((init_len - logo_len) / 2), " ")}{logo}{'#'.rjust(int((init_len - logo_len) / 2), " ")}
"""
        banner_dict = {
            "Python Version": platform.python_version(),
            "Python Compiler": platform.python_compiler(),
            "System Encoding": sys.getdefaultencoding(),
            "OS Version": platform.platform(),
            "CPU": platform.processor(),
            "Date Time": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            "Weekend": f"{datetime.now().strftime('%A')}",
            "Log Path": LogConfig.path,
            "Log Level": LogConfig.level,
            "Log Back Content": LogConfig.backup_count,
            "Log Max Bytes": LogConfig.max_bytes
        }

        for k, v in banner_dict.items():
            v_ = str(v)
            line = f"{k.ljust(key_len)}{'=>'.center(split_len, ' ')}{v_.rjust(value_len)}"
            content += f"{'#'.ljust(left_len, ' ')}{line}{'#'.rjust(init_len - len(line) - left_len, ' ')}\n"

        content += f"{''.center(init_len, '#')}"
        logger = self.get_logger()
        logger.info(content)

    def get_logger(self, name: str = "root") -> Logger:
        if name not in self.__logger_dict:
            self.__logger_dict[name] = _LoggerWrapper(name)
        return self.__logger_dict.get(name)


LoggerFactory = _LoggerFactory(True)

__all__ = ["LoggerFactory"]
